@@ -737,9 +737,9 @@ setMethod("getDesign", "unmarkedFrameOccuMS",
       return(c('p[1]','p[2]','delta'))
     }
     inds <- matrix(NA,nrow=S,ncol=S)
-    inds <- lower.tri(inds,diag=T)
+    inds <- lower.tri(inds,diag=TRUE)
     inds[,1] <- FALSE
-    inds <- which(inds,arr.ind=T) - 1
+    inds <- which(inds,arr.ind=TRUE) - 1
     paste0('p[',inds[,2],inds[,1],']')
   }
 
@@ -930,6 +930,7 @@ setMethod("getDesign", "unmarkedFramePCO",
     M <- nrow(y)
     T <- umf@numPrimary
     J <- ncol(y) / T
+    R <- obsNum(umf) / T
     delta <- umf@primaryPeriod
 
     if(is.null(umf@yearlySiteCovs))
@@ -955,18 +956,18 @@ setMethod("getDesign", "unmarkedFramePCO",
         Xlam.offset[is.na(Xlam.offset)] <- 0
 
     if(is.null(obsCovs(umf)))
-        obsCovs <- data.frame(placeHolder = rep(1, M*J*T))
+        obsCovs <- data.frame(placeHolder = rep(1, M*R*T))
     else
         obsCovs <- obsCovs(umf)
 
     colNames <- c(colnames(obsCovs), colnames(yearlySiteCovs))
 
     # Add yearlySiteCovs, which contains siteCovs
-    obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*T), each = J),])
+    obsCovs <- cbind(obsCovs, yearlySiteCovs[rep(1:(M*T), each = R),])
     colnames(obsCovs) <- colNames
 
     if(!("obsNum" %in% names(obsCovs)))
-        obsCovs <- cbind(obsCovs, obsNum = as.factor(rep(1:(J*T), M)))
+        obsCovs <- cbind(obsCovs, obsNum = as.factor(rep(1:(R*T), M)))
 
     # Ignore last year of data
     transCovs <- yearlySiteCovs[-seq(T, M*T, by=T),,drop=FALSE]
